#include "validation.h"

// Simulates the travel of a spacecraft across the points in space.
// It first prints the list of points, after which it will start receiving input.
// For every index of a point in space that the submission prints,
// we output the corresponding point in space that we actually travelled to thanks to the wormholes.
// The interactor tracks the amount of fuel used, and refuels when the submission flies back to the first point.
// If the submission runs out of fuel or refuels more than n/2 times, a WRONG_ANSWER verdict is given.

using namespace std;

typedef pair<long long, long long> pii;
typedef vector<long long> vi;
typedef vector<pair<pii, long long>> viii;

long double dist(pii a, pii b) {
    return sqrtl(
        (a.first - b.first) * (a.first - b.first) +
        (a.second - b.second) * (a.second - b.second)
    );
}

int main(int argc, char *argv[]) {
    // Set up the input and answer streams.
    std::ifstream in(argv[1]);
    OutputValidator v(argc, argv);

    long double d;
    int n, x, y;
    unsigned int seed;

    in >> n >> d >> seed;
    cout << n << " " << (long long) d << endl;

    viii points(n + 1);
    vi original_to_new_index(n + 1);

    for (int i = 1; i <= n; i++) {
        in >> x >> y;
        points[i] = make_pair(make_pair(x, y), i); // points[i].second is original_index
    }

    // Shuffle points 2 up to and including n - 1
    v.rng.seed(seed);
    Random::shuffle(next(next(points.begin())), prev(points.end()), v.rng);

    for (int i = 1; i <= n; i++) {
        auto [x, y] = points[i].first;
        cout << x << " " << y << endl;

        // original_to_new_index[original_index] = new_index;
        original_to_new_index[points[i].second] = i;
    }

    for (int i = 2; i < n; i++) {
        int original_index = points[i].second;
        // Map original_index to original_target, for example 2 -> 3, 3 -> 2, 4 -> 5, 5 -> 4, ...
        int original_target = original_index / 2 * 2 + 1 - original_index % 2;
        // points[i].second is new target
        points[i].second = original_to_new_index[original_target];
    }

    int interactions = 0;
    int refuels_left = n / 2;
    long double fuel_left = d;
    pii pos = points[1].first;
    while (true) {
        interactions++;
        int target = v.read_integer("target", 1, n);
        v.newline();

        fuel_left -= dist(pos, points[target].first);
        if (fuel_left < -1e-6 && fuel_left / d < -1e-6) {
            v.WA("Used ", -fuel_left, " too much fuel");
        }

        if (target == 1) {
            if (refuels_left <= 0) {
                v.WA("Refueling more than n / 2 times");
            }
            refuels_left--;
            fuel_left = d;
            cout << 1 << endl;
        } else if (target == n) {
            cerr << "Reached destination, stopping interaction" << endl;
            cerr << "Number of interactions: " << interactions << endl;
            return 0;
        } else {
            target = points[target].second; // Update target based on wormhole
            cout << target << endl;
        }
        pos = points[target].first;
    }
}
